/*
	(C) 2004 - 2008 Thomas Bleeker
	MadWizard.org
	
	Released under the BSD license with the exception of parts of the code 
	not written by Thomas Bleeker (www.madwizard.org), which are marked in
	the source code. See license.txt for details.
*/

#include "Dino.h"
#include "util.h"

Dino::Dino(void)
{
	_body.loadSGF(DINO_BODY_FILE);
	_eyes.loadSGF(DINO_EYES_FILE);
	_arm.loadSGF(DINO_ARM_FILE);
	_leg.loadSGF(DINO_LEG_FILE);

	// Load the texture
	CGTexture *pTex = new CGTexture(DINO_TEXTURE);
	if (pTex->loadTextureFile())
	{
		_body.setTexture(pTex, DINO_TEXTURE_BOX);
		_arm.setTexture(pTex, DINO_TEXTURE_BOX);
		_leg.setTexture(pTex, DINO_TEXTURE_BOX);
	}
	
	_phase = 0.0;

	_position =  Vertex3(0,0,0);
	_direction = Vector3(-1,0,1);
}

Dino::~Dino(void)
{
}

void Dino::draw()
{
	const float PI = 3.14159265358979324f;
	glPushMatrix();

		// Move dino to position
		glTranslatef(_position.x(), _position.y(), _position.z());

		// Center the dino at its position
		glTranslatef(-8, 0, 0);


		// Rotate the dinosaur into the right direction
		float angle = rad2deg(atan2(-_direction.z(), _direction.x()));
		rotateAround(8.0, 0.0,0, 0, 1, 0, angle);	

		// Move z-axis to the middle of the dino
		glTranslatef(0,0,-1.5);
			
		// Calculate some cycle values
		float	cosVal  =  cos(_phase * 2 * PI); // cosine function
		float   sinVal  =  sin(_phase * 2 * PI); // sine function
		float	cosVal2	=  cos((_phase+0.5) * 2 * PI); // second cosine function

		// Make the whole dino waggle a bit
		rotateAround(7.5, 0,   0, 0, 1, 0,  cosVal * 10);
		rotateAround(0,   1.5, 0, 1, 0, 0, -sinVal * 10);

		// Draw body and eyes at the right positions
		_body.draw();
		glTranslatef(0, 0, -0.1);
		_eyes.draw();
		glTranslatef(0, 0, 0.1);
	
		// Draw one leg	
		glPushMatrix();
			float up = getLegHeight(_phase);
			glTranslatef(0, up, 3);
			rotateAround(10, 5, 0,	0, 0, 1,		cosVal * -55);
			_leg.draw();
		glPopMatrix();
		
		// Draw other leg
		glPushMatrix();
			float q = fmod(_phase + 0.5, 1.0);
			up = getLegHeight(q);
			glTranslatef(0, up, -1.5);
			rotateAround(10, 5, 0,	0, 0, 1,		cosVal2 * -55);
			_leg.draw();
		glPopMatrix();
		
		// Draw one arm
		glPushMatrix();
			glTranslatef(0, 0, -0.6);
			rotateAround(10, 10, 0,		0, 0, 1,	cosVal2 * 20);
			_arm.draw();
		glPopMatrix();

		// Draw other arm
		glPushMatrix();
			glTranslatef(0, 0, 2.7);
			rotateAround(10, 10, 0,		0, 0, 1,	cosVal2 * -20);
			_arm.draw();
		glPopMatrix();

	glPopMatrix();


}

void Dino::rotateAround(float cx, float cy, float cz,
				  float ax, float ay, float az,
				  float angle)
{

	glTranslatef(cx, cy, cz);
	glRotatef(angle, ax, ay, az);
	glTranslatef(-cx, -cy, -cz);
}

float Dino::interpolate(float s, float e, float v, float d)
{
	// Interpolates sort of smoothly
	v = v / d;

	v = v * 3.14;
	v = v - 3.14/2;
	v = sin(v);
	v += 1.0;
	v /= 2.0;
	return v * (e-s) + s;
}

float Dino::getLegHeight(float phase)
{
	float up = 0;
	float top = 1.5;		// top position of the leg
	float center = 1;		// center position of the leg
	float bottom = 0;		// lowest position of the leg
	float step = 0.30;		// position when taking a step up
		
	if (phase < 0.25)
	{
		up = interpolate(center, top, phase, 0.25);
	}
	else if (phase < 0.5)
	{
		up = interpolate(top, bottom, phase - 0.25, 0.25);
	}
	else if (phase < 0.75)
	{
		up = interpolate(bottom, step, phase - 0.5, 0.25);
	}
	else //if (phase < 1.0)
	{
		up = interpolate(step, center, phase - 0.75, 0.25);
	}
	return up;
}
void Dino::setPosition(Vertex3 pos)
{
	_position = pos;
}

void Dino::setDirection(Vector3 dir)
{
	_direction = dir;
}

void Dino::setPhase(float phase)
{
	_phase = phase;
}